function OUTPUT2=conv_evenextension_demo_CRC;

% Demo of convolution in DCT domain
% h - convolution kernel
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), 
% OUTPUT2=conv_evenextension_demo_CRC;

SzX=128;
k=menu('Select convolution kernel','Rectangular impulse','Mexican hat','Ramp','Sinc-function','Integrator','Differentiator');
switch k
case 1, h=zeros(SzX,1);h(SzX/2+1-16:SzX/2+1+15)=1;
case 2, s=0:SzX-1;h=exp(-((s-65).^2)/(SzX));
case 3, h=zeros(128,1);h(65-16:65+15)=(1:32)-15.5;h=h/max(h);
case 4, h=zeros(SzX/8,1);h(SzX/16+1)=1;h=sincint_(h,8);h=[h(SzX);h(1:SzX-1)];
case 5, ;
    f_mask=1:fix(SzX/2);
    f_mask=-1./(2*pi*i*f_mask);
    f_mask=[0,f_mask,conj(fliplr(f_mask(1:fix(SzX/2)-1)))];
    f_mask(fix(SzX/2)+1)=1/(2*pi*SzX);
    h=real(fftshift(fft(f_mask)));
    h=real((fft(f_mask)));h=h/max(h);
case 6,
    h=zeros(SzX,1);h(SzX/2+1)=1;
    h=differentiator_1D(h);
end

m=menu('Select','Automatic continuous show','Manual show (to go on press a key)');
Sign=zeros(128,1); Sign(2:13)=rand(12,1)-0.5; Sign=dct(Sign);Sign=Sign/max(Sign);
figure_n(1);
OUTPUT1=zeros(SzX);
for x=1:SzX,
    INP=zeros(SzX,1);INP(x)=1;
    OUTPUT2=conv1d__dct(INP,h,0);
    figure(1);
    subplot(4,1,1);
    stem(Sign,'b.','LineWidth',2);
    hold on
    stem(INP,'r.','LineWidth',2);
    axis tight;grid off
    title(['Input signal: blue; Position of the filter window: red; x=',num2str(x)],'FontWeight','Bold');
    set(gca,'FontWeight','Bold')
    hold off

    subplot(4,1,2);
    stem([Sign;flipud(Sign)],'b.','LineWidth',2);
    hold on
    stem([INP;flipud(INP)],'r.','LineWidth',2);
    axis tight;
    grid off
    title(['Input symmetrized: blue; Position of the filter window: red; x=',num2str(x)],'FontWeight','Bold');
    set(gca,'FontWeight','Bold')
    hold off
    
    subplot(4,1,3);
    stem([Sign;flipud(Sign)],'b.','LineWidth',2);
    hold on
    stem(OUTPUT2,'r.','LineWidth',2);
    axis([1 256 min([Sign',OUTPUT2]) 2]);
    grid off;
    title('Convolution over the extended signal (Conv kernel: red)','FontWeight','Bold');
    set(gca,'FontWeight','Bold');
    hold off
    OUTPUT1(x,:)=OUTPUT2(1:SzX);
    subplot(4,1,4);
    stem(Sign,'b.','LineWidth',2);
    hold on
    stem([OUTPUT1(x,:)],'r.','LineWidth',2); 
    subplot(4,1,4);
    stem([zeros(SzX,1)],'b.');    
    axis([1 128 min([Sign',OUTPUT2]) 2]);
    xlabel('Sample index','FontWeight','Bold');
    grid off;
    title('Convolution result (Conv kernel red)','FontWeight','Bold');
    set(gca,'FontWeight','Bold')
    drawnow;
    hold off
    if m==2,pause;end    
end
%---------------------------------------------------------------------------------------
function OUTP=sincint_(INP,Lx);

% 1-D sinc interpolation with Lx-fold signal stretching
% (SzX/2+1)th sample of the mask is equal to cos(pi*Ux);
% NO SIGNAL EXTENSION
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTP=sincint(INP,Lx);

SzX =max(size(INP));
INP=reshape(INP,SzX,1);
OUTP=zeros(Lx*SzX,1);
Ux=1/(Lx);
M=zeros(SzX,1);
M(1)=1; M((SzX/2)+1)=exp(i*pi*Ux);
r=2:SzX/2; Wn=i*2*pi*Ux/SzX;
M(r)=exp(Wn*(r-1)); M(SzX+2-r)=conj(M(r));
spinput=fftnorm(INP);
for l=1:Lx,
	spinput=spinput.*M;
    out=real(ifftnorm([spinput(1:SzX/2);real(spinput(SzX/2+1));spinput(SzX/2+2:SzX)]));
    key=zeros(Lx,1);key(l)=1;
	OUTP=kron(out,key)+OUTP;
end

%---------------------------------------------------------------------------------------
function OUTPUT=conv1d__dct(INP,h,PlotFlag);

% 1-D convolution in DCT domain
% h-convolution kernel; symmetry point must be (SzX/2+1)
% Plot output if PlotFlag==1, 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il). Sept. 19, 2004 , Bellaterra 
% Call OUTPUT=conv1d_dct(INP,h,PlotFlag);

[SzX, SzY]=size(INP);
if SzX==1,INP=INP';[SzX,SzY]=size(INP);end;

h=reshape(h,SzX,SzY);
hh=fftshift([zeros(fix((SzX+1)/2),1);h;zeros(fix(SzX/2),1)]);
%hh=fftshift([h(1:SzX/2);zeros(SzX,1);h(SzX/2+1:SzX)]);
Mxx=ifft(hh)*2*SzX;
INP_ext=[INP;flipud(INP)];
sp_ext=sdft05__0(INP_ext,1);
OUTPUT=sdft05__0(sp_ext.*Mxx,-1);
%OUTPUT=real(OUTPUT);
OUTPUT=real(OUTPUT);

if PlotFlag==1,
stem(OUTPUT,'.');axis tight;grid
end


if SzY==1, OUTPUT=OUTPUT';end

%---------------------------------------------------------------------------

function OUT=sdft05__0(INP,sign);
% SDT(1/2,0)

[SzX,SzY]=size(INP);
r=0:SzX-1;
M=(exp(i*pi*r'/SzX))/sqrt(SzX);
if sign==1,
    OUT=ifft(INP).*M;
else
    OUT=fft(INP./M);
end
%------------------------------------------------------------------------
function display1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display1(INPIMG)

clf
colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;

